{% load i18n %}
<p>
{% blocktrans with name=site.name %}
You are receiving this email because you requested a password reset
for your user account at {{ name }}.
{% endblocktrans %}
</p>
<p>
{% blocktrans %}
Please go to the following page and choose a new password:
{% endblocktrans %}
</p>
<p><a href="{{ protocol }}://{{ site.domain }}{% url 'user_management_api:password_reset_confirm' uidb64=uid token=token %}">{{ protocol }}://{{ site.domain }}{% url 'user_management_api:password_reset_confirm' uidb64=uid token=token %}</a></p>
<p>
{% blocktrans with name=site.name %}
The {{ name }} team.
{% endblocktrans %}
</p>
